/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * http//www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is ART Ontology API.
 *
 * The Initial Developer of the Original Code is University of Roma Tor Vergata.
 * Portions created by University of Roma Tor Vergata are Copyright (C) 2009.
 * All Rights Reserved.
 *
 * The ART Ontology API were developed by the Artificial Intelligence Research Group
 * (art.uniroma2.it) at the University of Roma Tor Vergata
 * Current information about the ART Ontology API can be obtained at 
 * http//art.uniroma2.it/owlart
 *
 */

/*
 * Contributor(s): Armando Stellato stellato@info.uniroma2.it
 */
package it.uniroma2.art.owlart.models.impl;

import java.util.HashSet;
import java.util.Iterator;

import it.uniroma2.art.owlart.exceptions.ModelAccessException;
import it.uniroma2.art.owlart.exceptions.ModelUpdateException;
import it.uniroma2.art.owlart.model.ARTLiteral;
import it.uniroma2.art.owlart.model.ARTResource;
import it.uniroma2.art.owlart.model.ARTURIResource;
import it.uniroma2.art.owlart.model.NodeFilters;
import it.uniroma2.art.owlart.models.BaseRDFTripleModel;
import it.uniroma2.art.owlart.models.OWLModel;
import it.uniroma2.art.owlart.models.OWLReasoner;
import it.uniroma2.art.owlart.navigation.ARTLiteralIterator;
import it.uniroma2.art.owlart.navigation.ARTNodeIterator;
import it.uniroma2.art.owlart.navigation.ARTResourceIterator;
import it.uniroma2.art.owlart.navigation.ARTStatementIterator;
import it.uniroma2.art.owlart.navigation.ARTURIResourceIterator;
import it.uniroma2.art.owlart.navigation.RDFIteratorImpl;
import it.uniroma2.art.owlart.utilities.RDFIterators;
import it.uniroma2.art.owlart.vocabulary.OWL;
import it.uniroma2.art.owlart.vocabulary.RDF;
import it.uniroma2.art.owlart.vocabulary.RDFS;

public class OWLModelImpl extends RDFSModelImpl implements OWLModel, OWLReasoner {

	private boolean owlThingMaterialization;
	private boolean transitiveReasoning;
	private boolean inverseReasoning;
	private boolean symmetricReasoning;

	public OWLModelImpl(BaseRDFTripleModel baseRep) {
		super(baseRep);

		// REASONING CONFIGURATION
		if (baseRep instanceof OWLReasoner) {
			owlThingMaterialization = ((OWLReasoner) baseRep).supportsOWLThingMaterialization();
			transitiveReasoning = ((OWLReasoner) baseRep).supportsTransitiveProperties();
			inverseReasoning = ((OWLReasoner) baseRep).supportsInverseProperties();
			symmetricReasoning = ((OWLReasoner) baseRep).supportsSymmetricProperties();
		} else {
			owlThingMaterialization = false;
			transitiveReasoning = false;
			inverseReasoning = false;
			symmetricReasoning = false;
		}
	}

	/****************************
	 * /* ADD/REMOVE METHODS ***
	 ****************************/

	public void addAnnotationProperty(String propertyURI, ARTURIResource superProperty, ARTResource... graphs)
			throws ModelUpdateException {
		ARTURIResource newProp = baseRep.createURIResource(propertyURI);
		baseRep.addTriple(newProp, RDF.Res.TYPE, OWL.Res.ANNOTATIONPROPERTY, graphs);
		if (superProperty != null)
			baseRep.addTriple(newProp, RDFS.Res.SUBPROPERTYOF, superProperty, graphs);
	}

	public void addDatatypeProperty(String propertyURI, ARTURIResource superProperty, ARTResource... graphs)
			throws ModelUpdateException {
		ARTURIResource newProp = baseRep.createURIResource(propertyURI);
		baseRep.addTriple(newProp, RDF.Res.TYPE, OWL.Res.DATATYPEPROPERTY, graphs);
		if (superProperty != null)
			baseRep.addTriple(newProp, RDFS.Res.SUBPROPERTYOF, superProperty, graphs);
	}

	public void addObjectProperty(String propertyURI, ARTURIResource superProperty, ARTResource... graphs)
			throws ModelUpdateException {
		ARTURIResource newProp = baseRep.createURIResource(propertyURI);
		baseRep.addTriple(newProp, RDF.Res.TYPE, OWL.Res.OBJECTPROPERTY, graphs);
		if (superProperty != null)
			baseRep.addTriple(newProp, RDFS.Res.SUBPROPERTYOF, superProperty, graphs);
	}

	public void addOntologyProperty(String propertyURI, ARTURIResource superProperty, ARTResource... graphs)
			throws ModelUpdateException {
		ARTURIResource newProp = baseRep.createURIResource(propertyURI);
		baseRep.addTriple(newProp, RDF.Res.TYPE, OWL.Res.ONTOLOGYPROPERTY, graphs);
		if (superProperty != null)
			baseRep.addTriple(newProp, RDFS.Res.SUBPROPERTYOF, superProperty, graphs);
	}

	public void addImportStatement(String baseURI, ARTResource... contexts) throws ModelUpdateException {
		ARTURIResource importedRes = baseRep.createURIResource(baseURI);
		baseRep.addTriple(baseRep.createURIResource(getBaseURI()), OWL.Res.IMPORTS, importedRes, contexts);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see it.uniroma2.art.ontapi.repository.impl.ARTRDFSRepositoryImpl#addNewClass(java.lang.String)
	 * 
	 * while the RDFS adds a rdfs:Class, this one adds an owl:Class
	 */
	@Override
	public void addClass(String uri, ARTResource... contexts) throws ModelUpdateException {
		ARTURIResource newClass = baseRep.createURIResource(uri);
		baseRep.addTriple(newClass, RDF.Res.TYPE, OWL.Res.CLASS, contexts);
	}

	public void removeImportStatement(ARTURIResource URI, ARTResource... contexts)
			throws ModelUpdateException {
		baseRep.deleteTriple(baseRep.createURIResource(getBaseURI()), OWL.Res.IMPORTS, URI, contexts);
	}

	/**************************************
	 *** TRIPLE INSTANTIATION METHODS ***
	 **************************************/

	public void instantiateDatatypeProperty(ARTResource subject, ARTURIResource predicate,
			String literalString, ARTResource... contexts) throws ModelUpdateException {
		ARTLiteral lit = baseRep.createLiteral(literalString);
		baseRep.addTriple(subject, predicate, lit, contexts);
	}

	public void instantiateObjectProperty(ARTResource subject, ARTURIResource predicate, ARTResource object,
			ARTResource... contexts) throws ModelUpdateException {
		baseRep.addTriple(subject, predicate, object, contexts);
	}

	public void instantiateAnnotationProperty(ARTResource subject, ARTURIResource property, String value,
			String lang, ARTResource... contexts) throws ModelUpdateException {
		ARTLiteral lit = baseRep.createLiteral(value, lang);
		baseRep.addTriple(subject, property, lit, contexts);
	}

	/*************************
	 *** BOOLEAN METHODS ***
	 *************************/

	public boolean isAnnotationProperty(ARTURIResource prop, ARTResource... contexts)
			throws ModelAccessException {
		return baseRep.hasTriple(prop, RDF.Res.TYPE, OWL.Res.ANNOTATIONPROPERTY, true, contexts);
	}

	public boolean isDatatypeProperty(ARTURIResource prop, ARTResource... contexts)
			throws ModelAccessException {
		return baseRep.hasTriple(prop, RDF.Res.TYPE, OWL.Res.DATATYPEPROPERTY, true, contexts);
	}

	public boolean isOntologyProperty(ARTURIResource prop, ARTResource... graphs) throws ModelAccessException {
		return baseRep.hasTriple(prop, RDF.Res.TYPE, OWL.Res.ONTOLOGYPROPERTY, true, graphs);
	}

	public boolean isFunctionalProperty(ARTURIResource property, ARTResource... contexts)
			throws ModelAccessException {
		return baseRep.hasTriple(property, RDF.Res.TYPE, OWL.Res.FUNCTIONALPROPERTY, true, contexts);
	}

	public boolean isInverseFunctionalProperty(ARTURIResource property, ARTResource... contexts)
			throws ModelAccessException {
		return baseRep.hasTriple(property, RDF.Res.TYPE, OWL.Res.INVERSEFUNCTIONALPROPERTY, true, contexts);
	}

	public boolean isObjectProperty(ARTURIResource property, ARTResource... contexts)
			throws ModelAccessException {
		return baseRep.hasTriple(property, RDF.Res.TYPE, OWL.Res.OBJECTPROPERTY, true, contexts);
	}

	public boolean isSymmetricProperty(ARTURIResource property, ARTResource... contexts)
			throws ModelAccessException {
		return baseRep.hasTriple(property, RDF.Res.TYPE, OWL.Res.SYMMETRICPROPERTY, true, contexts);
	}

	public boolean isTransitiveProperty(ARTURIResource property, ARTResource... contexts)
			throws ModelAccessException {
		return baseRep.hasTriple(property, RDF.Res.TYPE, OWL.Res.TRANSITIVEPROPERTY, true, contexts);
	}

	public boolean isDataRange(ARTResource range, ARTResource... graphs) throws ModelAccessException {
		return baseRep.hasTriple(range, RDF.Res.TYPE, OWL.Res.DATARANGE, true, graphs);
	}

	public boolean isOntology(ARTURIResource uri, ARTResource... contexts) throws ModelAccessException {
		return baseRep.hasTriple(uri, RDF.Res.TYPE, OWL.Res.ONTOLOGY, true, contexts);
	}

	/*************************
	 *** LIST/GET METHODS ***
	 *************************/

	/*
	 * (non-Javadoc)
	 * 
	 * @see it.uniroma2.art.owlart.models.OWLModel#listAnnotationProperties(boolean,
	 * it.uniroma2.art.owlart.model.ARTResource[])
	 */
	public ARTURIResourceIterator listAnnotationProperties(boolean inferred, ARTResource... graphs)
			throws ModelAccessException {
		return RDFIterators.toURIResourceIterator(listSubjectsOfPredObjPair(RDF.Res.TYPE,
				OWL.Res.ANNOTATIONPROPERTY, inferred, graphs));
	}

	public ARTURIResourceIterator listDatatypeProperties(boolean inferred, ARTResource... graphs)
			throws ModelAccessException {
		return RDFIterators.toURIResourceIterator(listSubjectsOfPredObjPair(RDF.Res.TYPE,
				OWL.Res.DATATYPEPROPERTY, inferred, graphs));
	}

	public ARTURIResourceIterator listObjectProperties(boolean inferred, ARTResource... graphs)
			throws ModelAccessException {
		return RDFIterators.toURIResourceIterator(listSubjectsOfPredObjPair(RDF.Res.TYPE,
				OWL.Res.OBJECTPROPERTY, inferred, graphs));
	}

	public ARTURIResourceIterator listOntologyProperties(boolean inferred, ARTResource... graphs)
			throws ModelAccessException {
		return RDFIterators.toURIResourceIterator(listSubjectsOfPredObjPair(RDF.Res.TYPE,
				OWL.Res.ONTOLOGYPROPERTY, inferred, graphs));
	}

	public ARTURIResource getInverseProperty(ARTURIResource objectProperty, boolean inferred,
			ARTResource... graphs) throws ModelAccessException {
		ARTResourceIterator it = listSymmetricPropertyBoundConcepts(objectProperty, OWL.Res.INVERSEOF,
				inferred, graphs);
		if (it.streamOpen())
			return it.getNext().asURIResource();
		else
			return null;
	}

	public ARTURIResourceIterator listOntologyImports(ARTURIResource ontology, ARTResource... contexts)
			throws ModelAccessException {
		return new URIResourceIteratorWrappingNodeIterator(new ObjectsOfStatementsIterator(baseRep
				.listStatements(ontology, OWL.Res.IMPORTS, NodeFilters.ANY, false, contexts)));
	}

	public ARTLiteralIterator listValuesOfSubjDTypePropertyPair(ARTResource instance,
			ARTURIResource property, boolean inferred, ARTResource... contexts) throws ModelAccessException {
		return new LiteralIteratorWrappingNodeIterator(new ObjectsOfStatementsIterator(baseRep
				.listStatements(instance, property, NodeFilters.ANY, inferred, contexts)));
	}

	public ARTResourceIterator listValuesOfSubjObjPropertyPair(ARTResource individual,
			ARTURIResource property, boolean inferred, ARTResource... contexts) throws ModelAccessException {
		return new ResourceIteratorWrappingNodeIterator(new ObjectsOfStatementsIterator(baseRep
				.listStatements(individual, property, NodeFilters.ANY, inferred, contexts)));
	}

	// if infer==true then retrieves all rdfs classes, otherwise retrieves all owl:class TODO in non-infer
	// mode, should do some trivial owl+rdfs classes
	public ARTURIResourceIterator listNamedClasses(boolean inferred, ARTResource... graphs)
			throws ModelAccessException {
		if (inferred == true)
			return new URIResourceIteratorFilteringResourceIterator(new SubjectsOfStatementsIterator(baseRep
					.listStatements(NodeFilters.ANY, RDF.Res.TYPE, RDFS.Res.CLASS, inferred, graphs)));
		else
			return new URIResourceIteratorFilteringResourceIterator(new SubjectsOfStatementsIterator(baseRep
					.listStatements(NodeFilters.ANY, RDF.Res.TYPE, OWL.Res.CLASS, inferred, graphs)));
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see it.uniroma2.art.ontapi.repository.impl.ARTRDFSRepositoryImpl#retrieveClass(java.lang.String)
	 * 
	 * this implementation tries to makedo with the possibile lackness of a reasoner, so that owl classes
	 * would not be recognized as rdfs classes
	 */
	@Override
	public ARTURIResource retrieveClass(String uri, ARTResource... contexts) throws ModelAccessException {
		ARTStatementIterator statIt = baseRep.listStatements(baseRep.createURIResource(uri), RDF.Res.TYPE,
				RDFS.Res.CLASS, false, contexts);
		if (statIt.streamOpen()) {
			ARTURIResource cls = statIt.next().getSubject().asURIResource();
			statIt.close();
			return cls;
		}

		statIt = baseRep.listStatements(baseRep.createURIResource(uri), RDF.Res.TYPE, OWL.Res.CLASS, false,
				contexts);
		if (statIt.streamOpen()) {
			ARTURIResource cls = statIt.next().getSubject().asURIResource();
			statIt.close();
			return cls;
		}

		return null;
	}

	public ARTLiteralIterator parseDataRange(ARTResource dataRange, ARTResource... graphs)
			throws ModelAccessException {
		ARTResourceIterator resIt = listValuesOfSubjObjPropertyPair(dataRange, OWL.Res.ONEOF, true, graphs);

		if (resIt.streamOpen()) {
			ARTResource oneOfInstance = resIt.getNext();
			resIt.close();
			ARTNodeIterator it = RDFIterators.createRDFListIterator(this, oneOfInstance, true, graphs);
			return RDFIterators.toLiteralIterator(it);
		}
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @seeit.uniroma2.art.ontapi.repository.impl.ARTRDFSRepositoryImpl#isClass(it.uniroma2.art.ontapi.model.
	 * ARTResource)
	 * 
	 * this implementation tries to makedo with the possibile lackness of a reasoner, so that owl classes
	 * would not be recognized as rdfs classes
	 */
	@Override
	public boolean isClass(ARTResource cls, ARTResource... contexts) throws ModelAccessException {
		if (baseRep.hasTriple(cls, RDF.Res.TYPE, RDFS.Res.CLASS, true, contexts))
			return true;
		else
			return baseRep.hasTriple(cls, RDF.Res.TYPE, OWL.Res.CLASS, true, contexts);
	}

	/******************************
	 *** OWL REASONING METHODS ***
	 ******************************/

	public boolean supportsOWLThingMaterialization() {
		return owlThingMaterialization;
	}

	public boolean supportsInverseProperties() {
		return inverseReasoning;
	}

	public boolean supportsTransitiveProperties() {
		return transitiveReasoning;
	}

	public boolean supportsSymmetricProperties() {
		return symmetricReasoning;
	}

	/***********************************************
	 *** TRIVIAL OWL REASONING FACILITY METHODS ***
	 ***********************************************/
	// TODO the following methods can be improved by providing look-ahead iterators carrying an hashset with
	// them and filling it progressively (when calls to hasNext are made) to avoid repetitions, instead of
	// filling the whole set and then returning hashSet.iterator();

	/**
	 * this method is invoked by this OWLModel implementation when one needs to known exactly if a triple is
	 * available through inference on property which has an inverse. In this case, if a reasoner is not
	 * available or if it is not able to provide owl:inverseOf computation, then an algorithmic solution is
	 * adopted to provide on-the-fly computation for the property passed as an argument.
	 * 
	 * @param resource
	 * @param property
	 * @param inverseProperty
	 * @param object
	 * @param inferred
	 * @param graphs
	 * @return
	 * @throws ModelAccessException
	 */
	protected boolean hasInversePropertyBoundConcept(ARTResource resource, ARTURIResource property,
			ARTURIResource inverseProperty, ARTResource object, boolean inferred, ARTResource... graphs)
			throws ModelAccessException {

		boolean result = baseRep.hasTriple(resource, property, object, inferred, graphs);

		if (result || !inferred || supportsInverseProperties())
			return result;
		// if i found the result I'm already ok in any case, but if is not, and I want inference and my
		// reasoner does not support inverse properties, then...
		else {
			return baseRep.hasTriple(object, inverseProperty, resource, inferred, graphs);
		}
	}

	/**
	 * this method is invoked by this OWLModel implementation when one needs to known exactly if a triple is
	 * available through inference on property which is symmetric. In this case, if a reasoner is not
	 * available or if it is not able to provide owl:Symmetric computation, then an algorithmic solution is
	 * adopted to provide on-the-fly computation for the property passed as an argument.
	 * 
	 * @param resource
	 * @param property
	 * @param object
	 * @param inferred
	 * @param graphs
	 * @return
	 * @throws ModelAccessException
	 */
	protected boolean hasSymmetricPropertyBoundConcept(ARTResource resource, ARTURIResource property,
			ARTResource object, boolean inferred, ARTResource... graphs) throws ModelAccessException {

		// could also be implemented through hasInversePropertyBoundConcept by using same property as direct
		// and inverse of itself. Use this shortcut if further reengineering is needed

		boolean result = baseRep.hasTriple(resource, property, object, inferred, graphs);

		if (result || !inferred || supportsSymmetricProperties())
			return result;
		// if i found the result I'm already ok in any case, but if is not, and I want inference and my
		// reasoner does not support symmetric properties, then...
		else {
			return baseRep.hasTriple(object, property, resource, inferred, graphs);
		}
	}

	/**
	 * this method is invoked by this OWLModel implementation when one needs to known exactly if a triple is
	 * available through inference on a transitive property. In this case, if a reasoner is not available or
	 * if it is not able to provide owl:inverseOf computation, then an algorithmic solution is adopted to
	 * provide on-the-fly computation for the property passed as an argument.
	 * 
	 * @param resource
	 * @param property
	 * @param object
	 * @param inferred
	 * @param graphs
	 * @return
	 * @throws ModelAccessException
	 */
	protected boolean hasTransitivePropertyBoundConcept(ARTResource resource, ARTURIResource property,
			ARTResource object, boolean inferred, ARTResource... graphs) throws ModelAccessException {
		return hasTransitivePropertyBoundConcept(resource, property, property, object, inferred, graphs);
	}

	/**
	 * * as for
	 * {@link #hasTransitivePropertyBoundConcept(ARTResource, ARTURIResource, ARTResource, boolean, ARTResource...)}
	 * though the transitive property is the transitive extension of a non-transitive property.<br/>
	 * If no reasoning is asked, the non-transitive property is looked in the concrete triples.<br/>
	 * In case of reasoning (inferred==true), if a reasoner is able to compute it (it has transitive property
	 * reasoning), the transitive property is explored, while if it is not, a recursive exploration of the
	 * non-transitive property is conducted
	 * 
	 * @param resource
	 * @param nonTransitiveProperty
	 * @param transitiveExtensionProperty
	 * @param object
	 * @param inferred
	 * @param graphs
	 * @return
	 * @throws ModelAccessException
	 */
	protected boolean hasTransitivePropertyBoundConcept(ARTResource resource,
			ARTURIResource nonTransitiveProperty, ARTURIResource transitiveExtensionProperty,
			ARTResource object, boolean inferred, ARTResource... graphs) throws ModelAccessException {
		if (!inferred)
			return hasTriple(resource, nonTransitiveProperty, object, false, graphs);
		if (supportsTransitiveProperties()) {
			return hasTriple(resource, transitiveExtensionProperty, object, true, graphs);
		} else {
			HashSet<ARTResource> set = new HashSet<ARTResource>();
			return recursiveCheckExploration(set, resource, nonTransitiveProperty, object, graphs);
		}
	}

	/**
	 * @param resource
	 * @param nonTransitiveProperty
	 * @param nonTransitiveInverseProperty
	 *            inverse property of property <code>nonTransitiveProperty</code>
	 * @param transitiveExtensionProperty
	 *            transitive extension of property <code>nonTransitiveProperty</code>
	 * @param transitiveExtensionInverseProperty
	 *            transitive extension of property <code>nonTransitiveInverseProperty</code>
	 * @param object
	 * @param inferred
	 * @param graphs
	 * @return
	 * @throws ModelAccessException
	 */
	protected boolean hasTransitiveInversePropertyBoundConcept(ARTResource resource,
			ARTURIResource nonTransitiveProperty, ARTURIResource nonTransitiveInverseProperty,
			ARTURIResource transitiveExtensionProperty, ARTURIResource transitiveExtensionInverseProperty,
			ARTResource object, boolean inferred, ARTResource... graphs) throws ModelAccessException {

		if (!inferred)
			return hasTriple(resource, nonTransitiveProperty, object, false, graphs);

		if (supportsTransitiveProperties() && supportsInverseProperties()) {
			return hasTriple(resource, transitiveExtensionProperty, object, true, graphs);
		} else if (supportsTransitiveProperties()) {
			return hasInversePropertyBoundConcept(resource, transitiveExtensionProperty,
					transitiveExtensionInverseProperty, object, true, graphs);
		} else if (supportsInverseProperties()) {
			return hasTransitivePropertyBoundConcept(resource, nonTransitiveProperty,
					transitiveExtensionProperty, object, true, graphs);
		} else {
			HashSet<ARTResource> set = new HashSet<ARTResource>();
			return recursiveInversePropertiesCheckExploration(set, resource, nonTransitiveProperty,
					nonTransitiveInverseProperty, object, graphs);
		}
	}

	/**
	 * method which is invoked by this OWLModel implementation when the complete set of results from a
	 * property which has an inverse has to be obtained. In this case, if a reasoner is not available or if it
	 * is not able to provide owl:inverseOf computation, then an algorithmic solution is adopted to provide
	 * on-the-fly computation for the property passed as an argument.
	 * 
	 * @param resource
	 * @param property
	 * @param inverseProperty
	 * @param graphs
	 * @return
	 * @throws ModelAccessException
	 */
	protected ARTResourceIterator listInversePropertyBoundConcepts(ARTResource resource,
			ARTURIResource property, ARTURIResource inverseProperty, boolean inferred, ARTResource... graphs)
			throws ModelAccessException {

		ARTResourceIterator it = listValuesOfSubjObjPropertyPair(resource, property, inferred, graphs);

		if (!inferred || supportsInverseProperties()) {
			return it;
		} else {
			HashSet<ARTResource> set = new HashSet<ARTResource>();

			while (it.streamOpen()) {
				set.add(it.getNext());
			}
			it.close();

			it = listSubjectsOfPredObjPair(inverseProperty, resource, true, graphs);

			while (it.streamOpen()) {
				set.add(it.getNext());
			}
			it.close();

			return new ARTResourceIteratorImpl(set.iterator());

		}
	}

	/**
	 * method which is invoked by this OWLModel implementation when the complete set of results from a
	 * property which is symmetric has to be obtained. In this case, if a reasoner is not available or if it
	 * is not able to provide owl:Symmetric computation, then an algorithmic solution is adopted to provide
	 * on-the-fly computation for the property passed as an argument.
	 * 
	 * @param resource
	 * @param property
	 * @param graphs
	 * @return
	 * @throws ModelAccessException
	 */
	protected ARTResourceIterator listSymmetricPropertyBoundConcepts(ARTResource resource,
			ARTURIResource property, boolean inferred, ARTResource... graphs) throws ModelAccessException {
		return listInversePropertyBoundConcepts(resource, property, property, inferred, graphs);
	}

	/**
	 * method which is invoked by this OWLModel implementation when the complete set of results from a
	 * transitive property has to be obtained. In this case, if a reasoner is not available or if it is not
	 * able to provide owl:inverseOf computation, then an algorithmic solution is adopted to provide
	 * on-the-fly computation for the property passed as an argument.
	 * 
	 * @param resource
	 * @param property
	 * @param graphs
	 * @return
	 * @throws ModelAccessException
	 */
	protected ARTResourceIterator listTransitivePropertyBoundConcepts(ARTResource resource,
			ARTURIResource property, boolean inferred, ARTResource... graphs) throws ModelAccessException {
		return listTransitivePropertyBoundConcepts(resource, property, property, inferred, graphs);
	}

	/**
	 * as for
	 * {@link #listTransitivePropertyBoundConcepts(ARTResource, ARTURIResource, ARTURIResource, boolean, ARTResource...)}
	 * though the transitive property is the transitive extension of a non-transitive property.<br/>
	 * If no reasoning is asked, the non-transitive property is looked in the concrete triples.<br/>
	 * In case of reasoning (inferred==true), if a reasoner is able to compute it (it has transitive property
	 * reasoning), the transitive property is explored, while if it is not, a recursive exploration of the
	 * non-transitive property is conducted
	 * 
	 * @param resource
	 * @param nonTransitiveProperty
	 * @param transitiveExtensionProperty
	 *            transitive extension of property <code>property</code>
	 * @param graphs
	 * @return
	 * @throws ModelAccessException
	 */
	protected ARTResourceIterator listTransitivePropertyBoundConcepts(ARTResource resource,
			ARTURIResource nonTransitiveProperty, ARTURIResource transitiveExtensionProperty,
			boolean inferred, ARTResource... graphs) throws ModelAccessException {

		if (!inferred)
			return listValuesOfSubjObjPropertyPair(resource, nonTransitiveProperty, inferred, graphs);
		if (supportsTransitiveProperties()) {
			return listValuesOfSubjObjPropertyPair(resource, transitiveExtensionProperty, inferred, graphs);
		} else {
			HashSet<ARTResource> set = new HashSet<ARTResource>();
			recursiveExploration(set, resource, nonTransitiveProperty, graphs);
			return new ARTResourceIteratorImpl(set.iterator());

		}
	}

	/**
	 * 
	 * @param resource
	 * @param nonTransitiveProperty
	 * @param transitiveExtensionProperty
	 *            transitive extension of property <code>property</code>
	 * @param graphs
	 * @return
	 * @throws ModelAccessException
	 */
	protected ARTResourceIterator listTransitiveInversePropertyBoundConcepts(ARTResource resource,
			ARTURIResource nonTransitiveProperty, ARTURIResource nonTransitiveInverseProperty,
			ARTURIResource transitiveExtensionProperty, ARTURIResource transitiveExtensionInverseProperty,
			boolean inferred, ARTResource... graphs) throws ModelAccessException {

		if (!inferred)
			return listValuesOfSubjObjPropertyPair(resource, nonTransitiveProperty, false, graphs);

		if (supportsTransitiveProperties() && supportsInverseProperties()) {
			return listValuesOfSubjObjPropertyPair(resource, transitiveExtensionProperty, true, graphs);
		} else if (supportsTransitiveProperties()) {
			return listInversePropertyBoundConcepts(resource, transitiveExtensionProperty,
					transitiveExtensionInverseProperty, true, graphs);
		} else if (supportsInverseProperties()) {
			return listTransitivePropertyBoundConcepts(resource, nonTransitiveProperty,
					transitiveExtensionProperty, true, graphs);
		} else {
			HashSet<ARTResource> set = new HashSet<ARTResource>();
			recursiveInversePropertiesExploration(set, resource, nonTransitiveProperty,
					nonTransitiveInverseProperty, graphs);
			return new ARTResourceIteratorImpl(set.iterator());

		}
	}

	/**
	 * recursively explore (that is, runs along the transitive closure of) a property and fills the set
	 * <code>set</code> with all the objects of his property
	 * 
	 * @param set
	 * @param resource
	 * @param prop
	 * @param graphs
	 * @throws ModelAccessException
	 */
	private void recursiveExploration(HashSet<ARTResource> set, ARTResource resource, ARTURIResource prop,
			ARTResource... graphs) throws ModelAccessException {
		ARTResourceIterator it = listValuesOfSubjObjPropertyPair(resource, prop, true, graphs);
		while (it.streamOpen()) {
			ARTResource newres = it.getNext();
			if (!set.contains(newres)) {
				set.add(newres);
				recursiveExploration(set, newres, prop, graphs);
			}
		}
		it.close();
	}

	/**
	 * recursively explore (that is, runs along the transitive closure of) a property to find a triple with
	 * object = <code>object</code>
	 * 
	 * @param set
	 * @param resource
	 * @param prop
	 * @param graphs
	 * @throws ModelAccessException
	 */
	private boolean recursiveCheckExploration(HashSet<ARTResource> set, ARTResource resource,
			ARTURIResource prop, ARTResource object, ARTResource... graphs) throws ModelAccessException {
		boolean result = false;
		ARTResourceIterator it = listValuesOfSubjObjPropertyPair(resource, prop, true, graphs);
		while (it.streamOpen() && (result == false)) {
			ARTResource newres = it.getNext();
			if (!set.contains(newres)) {
				if (newres.equals(object)) {
					return true;
				} else {
					set.add(newres);
					result = recursiveCheckExploration(set, newres, prop, object, graphs);
				}
			}
		}
		it.close();
		return result;
	}

	/**
	 * recursively explore (that is, runs along the transitive closure of) a property, which has also a
	 * inverse property. So, for each node A, it explores all nodes reachable from A through property
	 * <code>prop</code> and all nodes reaching A through property <code>inverseProp</code> (the set of them
	 * is computed, that is, no repetitions), then for each of the nodes in the computed set, this process is
	 * applied again on them.
	 * 
	 * @param set
	 * @param resource
	 * @param prop
	 * @param inverseProp
	 * @param graphs
	 * @throws ModelAccessException
	 */
	private void recursiveInversePropertiesExploration(HashSet<ARTResource> set, ARTResource resource,
			ARTURIResource prop, ARTURIResource inverseProp, ARTResource... graphs)
			throws ModelAccessException {
		ARTResourceIterator it = listInversePropertyBoundConcepts(resource, prop, inverseProp, true, graphs);
		while (it.streamOpen()) {
			ARTResource newres = it.getNext();
			if (!set.contains(newres)) {
				set.add(newres);
				recursiveInversePropertiesExploration(set, newres, prop, inverseProp, graphs);
			}
		}
		it.close();
	}

	/**
	 * recursively explore (that is, runs along the transitive closure of) a property, which has also a
	 * inverse property, to find a triple with object == <code>object</code>. So, for each node A, it explores
	 * all nodes reachable from A through property <code>prop</code> and all nodes reaching A through property
	 * <code>inverseProp</code> (the set of them is computed, that is, no repetitions), then for each of the
	 * nodes in the computed set, this process is applied again on them, until <code>object</code> is found or
	 * the search is exhausted.
	 * 
	 * @param set
	 * @param resource
	 * @param prop
	 * @param inverseProp
	 * @param graphs
	 * @throws ModelAccessException
	 */
	private boolean recursiveInversePropertiesCheckExploration(HashSet<ARTResource> set,
			ARTResource resource, ARTURIResource prop, ARTURIResource inverseProp, ARTResource object,
			ARTResource... graphs) throws ModelAccessException {
		System.out
				.println("exploring: " + resource + " on props: " + prop + "|" + inverseProp + "|" + object);
		boolean result = false;
		ARTResourceIterator it = listInversePropertyBoundConcepts(resource, prop, inverseProp, true, graphs);
		while (it.streamOpen() && (result == false)) {
			ARTResource newres = it.getNext();
			System.out.println("next:" + newres);
			if (!set.contains(newres)) {
				if (newres.equals(object)) {
					return true;
				} else {
					set.add(newres);
					result = recursiveInversePropertiesCheckExploration(set, newres, prop, inverseProp,
							object, graphs);
				}
			}
		}
		it.close();
		return result;
	}

	protected class ARTResourceIteratorImpl extends RDFIteratorImpl<ARTResource> implements
			ARTResourceIterator {

		Iterator<ARTResource> it;

		ARTResourceIteratorImpl(Iterator<ARTResource> it) {
			this.it = it;
		}

		public void close() throws ModelAccessException {
		}

		public ARTResource getNext() {
			return it.next();
		}

		public boolean streamOpen() {
			return it.hasNext();
		}

	}

	protected class ARTLiteralIteratorImpl extends RDFIteratorImpl<ARTLiteral> implements ARTLiteralIterator {

		Iterator<ARTLiteral> it;

		ARTLiteralIteratorImpl(Iterator<ARTLiteral> it) {
			this.it = it;
		}

		public void close() throws ModelAccessException {
		}

		public ARTLiteral getNext() {
			return it.next();
		}

		public boolean streamOpen() {
			return it.hasNext();
		}

	}

}
